#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <sqlite3.h>
#include <string.h>
#include <math.h>

#include "include/db.h"

static void open_db(sqlite3** db)
{
	int err;

	err = sqlite3_open("books.db", db);
	if (err != SQLITE_OK) {
		fprintf(stderr, "Cannot open db: %s\n", sqlite3_errmsg(*db));
		sqlite3_close(*db);
		exit(EXIT_FAILURE);
	}
}

static int get_num_rows(sqlite3* db)
{
	int rows = 0;

	sqlite3_stmt* msg;
	sqlite3_prepare(db, "SELECT * FROM books;", -1, &msg, NULL);
	while (sqlite3_step(msg) == SQLITE_ROW)
		rows++;
	sqlite3_finalize(msg);
	
	return rows;
}

void setup_db(sqlite3* db)
{
	int err;
	char* err_buf = 0;

	open_db(&db);

	err = sqlite3_exec(db,
			"CREATE TABLE IF NOT EXISTS books (id UNSIGNED INT PRIMARY KEY, \
			title TEXT, subtitle TEXT, isbn TEXT, authors TEXT, subjects TEXT, \
			langs TEXT, date_added TEXT, pub_date TEXT, \
			image_code TEXT, year_of_pub YEAR, page_len UNSIGNED INT);",
			0, 0, &err_buf);
	if (err != SQLITE_OK) {
		fprintf(stderr, "sqlite3_exec: %s\n", err_buf);
		sqlite3_free(err_buf);
		sqlite3_close(db);
		exit(EXIT_FAILURE);
	}

	sqlite3_close(db);
}

static void add_str_entry(char** dst, char** src, bool last)
{
	size_t len;
	len = strlen(*src) + strlen(*dst) + 2 + 1 + (last ? 0 : 2);

	*dst = realloc(*dst, sizeof(char) * len);

	*dst = strcat(*dst, "\"");
	*dst = strcat(*dst, *src);
	*dst = strcat(*dst, "\"");
	if (!last)
		*dst = strcat(*dst, ", ");
}

static void add_int_entry(char** dst, int src, bool last)
{
	size_t len_int, len;
	char* num;

	len_int = (src == 0) ? 1 : floor(log10(abs(src))) + 1;

	num = malloc(sizeof(char) * (len_int + 1));
	snprintf(num, len_int + 1, "%d", src);

	len = strlen(*dst) + len_int + 1 + (last ? 0 : 2);
	*dst = realloc(*dst, sizeof(char) * len);
	*dst = strcat(*dst, num);

	if (!last)
		strcat(*dst, ", ");

	free(num);
}

static void string_cat_vals(char** dst, book_t* book, int id)
{
	add_int_entry(dst, id, false);
	add_str_entry(dst, &(book->title), false);
	add_str_entry(dst, &(book->subtitle), false);
	add_str_entry(dst, &(book->isbn), false);
	add_str_entry(dst, &(book->authors), false);
	add_str_entry(dst, &(book->subjects), false);
	add_str_entry(dst, &(book->langs), false);
	add_str_entry(dst, &(book->date_added), false);
	add_str_entry(dst, &(book->pub_date), false);
	add_str_entry(dst, &(book->image_code), false);
	add_int_entry(dst, book->page_len, true);

	*dst = realloc(*dst, sizeof(char) * (strlen(*dst) + 3));
	strcat(*dst, ");");
}

#define INSERT_STR "INSERT INTO books (id, title, subtitle, isbn, authors, subjects, langs, date_added, pub_date, image_code, page_len) VALUES("

void add_to_db(book_t* book, sqlite3* db)
{
	int id, err;
	char* cmd;
	char* err_msg = 0;

	open_db(&db);

	id = get_num_rows(db) + 4;
	
	cmd = malloc(sizeof(char) * (strlen(INSERT_STR) + 1));
	snprintf(cmd, strlen(INSERT_STR) + 1, "%s", INSERT_STR);

	string_cat_vals(&cmd, book, id);

	printf("%s\n", cmd);

	err = sqlite3_exec(db, cmd, 0, 0, &err_msg);
	if (err != SQLITE_OK) {
		fprintf(stderr, "sqlite3_exec: %s\n", err_msg);
		sqlite3_free(err_msg);
		sqlite3_close(db);
		exit(EXIT_FAILURE);
	}

	sqlite3_close(db);
	free(cmd);
}
#undef INSERT_STR

#define DEL_STR "DELETE FROM books WHERE id = "
void remove_from_db(int id, sqlite3* db)
{
	int err;
	char *cmd, *err_msg;

	open_db(&db);

	cmd = malloc(sizeof(char) * (strlen(DEL_STR) + 1));
	snprintf(cmd, strlen(DEL_STR), "%s", DEL_STR);

	add_int_entry(&cmd, id, true);

	err = sqlite3_exec(db, cmd, 0, 0, &err_msg);
	if (err != SQLITE_OK) {
		fprintf(stderr, "sqlite3_exec: %s\n", err_msg);
		sqlite3_free(err_msg);
		sqlite3_close(db);
		exit(EXIT_FAILURE);
	}

	sqlite3_close(db);
	free(cmd);
}

#undef DEL_STR
